row_key = "Rows"
column_key = "Cols"
special_keywords = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov',
                    'dec', 'january', 'february', 'march', 'april', 'may', 'june', 'july', 'august', 'september',
                    'october', 'november', 'december', 'q1', 'q2', 'q3', 'q4']

# cell classes
text_cell = "text"
number_cell = "number"
empty_cell = "empty"
date_cell = "date"
blank_cell = "blank"

row_orientation = "row"
column_orientation = "col"


def data_to_string(data):
    return str(data)
